@@ -20,6 +20,7 @@ from systemdlint.cls.helper import Helper
 from systemdlint.conf.knownPaths import KNOWN_PATHS
 from systemdlint.conf.knownUnits import KNOWN_RUNTIME_UNITS
 from systemdlint.conf.knownUnits import KNOWN_UNITS_EXT
+from systemdlint.conf.knownUnits import KNOWN_GENERATED_UNITS
 
 
 class Value(object):
@@ -102,7 +103,7 @@ class NumericValue(Value):
         val2 = val.lstrip("-")
         charSet = string.digits
         if self.__numberBase == 16:
-            charSet = string.hexdigits
+            charSet = string.hexdigits + "x"
         elif self.__numberBase == 8:
             charSet = string.octdigits
         if val2 in self.__specials:
@@ -117,7 +118,7 @@ class NumericValue(Value):
             val = self.__getPlainValue(value)
             if val in self.__specials:
                 return res
-            x = int(val)
+            x = int(val, self.__numberBase)
             if x % self.Base != 0:
                 res.append(ErrorInvalidNumericBase(
                     item.Key, self.Base, item.Line, item.File))
@@ -128,29 +129,51 @@ class NumericValue(Value):
     def GetAllowedValues(self, baseOnly=False):
         res = []
         _suffixess = self.__suffixes or [""]
+        __lower = self.__boundaries[0]
+        __upper = self.__boundaries[-1]
+        if self.__numberBase == 16:
+            __lower = hex(__lower)
+            __upper = hex(__upper)
+        if self.__numberBase == 8:
+            __lower = oct(__lower)
+            __upper = oct(__upper)
         if not baseOnly:
             for x in _suffixess:
-                res.append("{}{}".format(self.__boundaries[0], x))
-                res.append("{}{}".format(self.__boundaries[-1], x))
+                res.append("{}{}".format(__lower, x))
+                res.append("{}{}".format(__upper, x))
             for x in self.__specials:
                 res.append(x)
         else:
             if self.Base != 1:
                 for i in range(self.Base, self.Base * 10, self.Base):
-                    res.append(i)
+                    if self.__numberBase == 16:
+                        res.append(hex(i))
+                    if self.__numberBase == 8:
+                        res.append(oct(i))
         return res
 
     def GetInvalidValues(self, baseOnly=False):
         res = []
         _suffixess = self.__suffixes or [""]
+        __lower = self.__boundaries[0] - 1
+        __upper = self.__boundaries[-1] + 1
+        if self.__numberBase == 16:
+            __lower = hex(__lower)
+            __upper = hex(__upper)
+        if self.__numberBase == 8:
+            __lower = oct(__lower)
+            __upper = oct(__upper)
         if not baseOnly:
             for x in _suffixess:
-                res.append("{}{}".format(self.__boundaries[0] - 1, x))
-                res.append("{}{}".format(self.__boundaries[-1] + 1, x))
+                res.append("{}{}".format(__lower, x))
+                res.append("{}{}".format(__upper, x))
         else:
             if self.Base != 1:
                 for i in range(self.Base, self.Base * 10, self.Base):
-                    res.append(i + 1)
+                    if self.__numberBase == 16:
+                        res.append(hex(i + 1))
+                    if self.__numberBase == 8:
+                        res.append(oct(i + 1))
         return res
 
 
@@ -218,6 +241,28 @@ class EmptyValue(Value):
     def GetInvalidValues(self):
         return ["a"]
 
+class ListOf(Value):
+    def __init__(self, obj, delimiter=" ", conditional={}):
+        super().__init__(conditional)
+        self.__object = obj
+        self.__delimiter = delimiter
+
+    def IsAllowedValue(self, value):
+        return all(self.__object.IsAllowedValue(x) for x in self.CleanValue(value).split(self.__delimiter) if x)
+
+    def GetAllowedValues(self):
+        if any(self.__object.GetAllowedValues()):
+            return [self.__delimiter.join(self.__object.GetAllowedValues())]
+        return []
+
+    def GetInvalidValues(self):
+        res = []
+        if any(self.__object.GetInvalidValues()):
+            res = [self.__delimiter.join([str(x) for x in self.__object.GetInvalidValues()])]
+            if any(self.__object.GetAllowedValues()):
+                _x = [str(x) for x in self.__object.GetAllowedValues()] + [str(self.__object.GetInvalidValues()[0])]
+                res.append(self.__delimiter.join(_x))
+        return res
 
 class IPValue(Value):
     def __init__(self, conditional={}):
@@ -508,6 +553,25 @@ class OctalModeValue(Value):
     def GetInvalidValues(self):
         return [True, "999", "888"]
 
+class MacAddressValue(Value):
+    def __init__(self, conditional={}):
+        super().__init__(conditional)
+
+    def IsAllowedValue(self, value):
+        if any(re.match(x, self.CleanValue(value)) for x in [
+                                r"([a-f0-9]{2}-){5}[a-f0-9]{2}",
+                                r"([a-f0-9]{2}:){5}[a-f0-9]{2}",
+                                r"([A-F0-9]{4}\.){2}[A-F0-9]{4}",
+                                ]):
+            return True
+        return False
+
+    def GetAllowedValues(self):
+        return ["01:23:45:67:89:ab", "00-11-22-33-44-55", "AABB.CCDD.EEFF"]
+
+    def GetInvalidValues(self):
+        return [True, "01:23:45:67:89", "00-11-22-GG-44-55", "AABB:CCDD:EEFF"]
+
 
 class UrlListValue(Value):
     def __init__(self, conditional={}):
@@ -566,6 +630,10 @@ class UnitListValue(Value):
                         Helper.GetPath(args.rootpath, p))])
                     if found:
                         break
+            if any(re.match(x, k) for x in KNOWN_GENERATED_UNITS):
+                # filter out known generated units
+                # they are likely unavailable offline
+                found = True
             if not found:
                 res.append(ErrorRefUnitNotFound(k, item.Line, item.File))
         if self.__inverseMode:
